@@ -1,14 +1,16 @@
 from argparse import ArgumentParser
 from collections import OrderedDict
 from copy import deepcopy
+from functools import partial
 import json
 from pprint import pformat
+from shlex import quote
 import sys
 import time
-from warnings import warn
 from types import MethodType
 from typing import Any, Callable, Dict, List, Optional, Sequence, Set, Tuple, TypeVar, Union, get_type_hints
 from typing_inspect import is_literal_type, get_args
+from warnings import warn
 
 from tap.utils import (
     get_class_variables,
@@ -126,6 +128,9 @@ class Tap(ArgumentParser):
         # Get variable name
         variable = get_argument_name(*name_or_flags)
 
+        if self._underscores_to_dashes:
+            variable = variable.replace('-', '_')
+
         # Get default if not specified
         if hasattr(self, variable):
             kwargs['default'] = kwargs.get('default', getattr(self, variable))
@@ -188,7 +193,8 @@ class Tap(ArgumentParser):
                       and len(get_args(var_type)) > 0
                       and is_literal_type(get_args(var_type)[0])):
                     var_type, kwargs['choices'] = get_literals(get_args(var_type)[0], variable)
-                    kwargs['nargs'] = kwargs.get('nargs', '*')
+                    if kwargs.get('action') not in {'append', 'append_const'}:
+                        kwargs['nargs'] = kwargs.get('nargs', '*')
                 # Handle Tuple type (with type args) by extracting types of Tuple elements and enforcing them
                 elif get_origin(var_type) in (Tuple, tuple) and len(get_args(var_type)) > 0:
                     loop = False
@@ -239,7 +245,9 @@ class Tap(ArgumentParser):
                     kwargs['type'] = var_type
 
         if self._underscores_to_dashes:
-            name_or_flags = [name_or_flag.replace('_', '-') for name_or_flag in name_or_flags]
+            # Replace "_" with "-" for arguments that aren't positional
+            name_or_flags = tuple(name_or_flag.replace('_', '-') if name_or_flag.startswith('-') else name_or_flag
+                                  for name_or_flag in name_or_flags)
 
         super(Tap, self).add_argument(*name_or_flags, **kwargs)
 
@@ -291,7 +299,7 @@ class Tap(ArgumentParser):
 
         # Load each subparser
         for flag, subparser_type, kwargs in self._subparser_buffer:
-            self._subparsers._parser_class = subparser_type
+            self._subparsers._parser_class = partial(subparser_type, underscores_to_dashes=self._underscores_to_dashes)
             self._subparsers.add_parser(flag, **kwargs)
 
     def add_subparsers(self, **kwargs) -> None:
@@ -345,7 +353,7 @@ class Tap(ArgumentParser):
         :return: A dictionary of reproducibility information.
         """
         reproducibility = {
-            'command_line': f'python {" ".join(sys.argv)}',
+            'command_line': f'python {" ".join(quote(arg) for arg in sys.argv)}',
             'time': time.strftime('%c')
         }
 
@@ -660,3 +668,16 @@ class Tap(ArgumentParser):
             copied.__dict__[k] = deepcopy(v, memo)
 
         return copied
+
+    def __getstate__(self) -> Dict[str, Any]:
+        """Gets the state of the object for pickling."""
+        return self.as_dict()
+
+    def __setstate__(self, d: Dict[str, Any]) -> None:
+        """
+        Initializes the object with the provided dictionary of arguments for unpickling.
+
+        :param d: A dictionary of arguments.
+        """
+        self.__init__()
+        self.from_dict(d)
